// PluginSample.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "resource.h"

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    return TRUE;
}


void FilterAbout(LPFILTER_MODULE lpf, HWND hWnd)
{
	TCHAR szBuf[256];
	wsprintf(szBuf, _T("%s sample filter  2005 Conduits Technologies, Inc."),
		lpf->szMenu);
	MessageBox(hWnd, szBuf, _T("About"), MB_OK);
}

void FilterInit(LPFILTER_MODULE)
{
}

void FilterQuit(LPFILTER_MODULE)
{
}

BOOL FilterDefaultStartup(LPFILTER_MODULE lpf, int type)
{
	return TRUE;
}

BOOL FilterDefaultConfigure(LPFILTER_MODULE lpf, HWND hwndParent)
{
	return TRUE;
}


double g_ramp[3][256];

void FilterGrayscaleInit(LPFILTER_MODULE)
{
	for (int i = 0; i < 256; i++) {
		g_ramp[0][i] = 0.2126 * pow(i/255.0, 2.2);
		g_ramp[1][i] = 0.7152 * pow(i/255.0, 2.2);
		g_ramp[2][i] = 0.0724 * pow(i/255.0, 2.2);
	}
}



BOOL FilterGrayscaleExecute(LPFILTER_MODULE lpf, BOOL bStart)
{
	if (lpf->Canceled()) return FALSE;

	if (!IsRectEmpty(&lpf->rcIn)) {
		// process data @ rcIn, one row at a time
		BYTE* data = lpf->pixels.data;
		for (int i = lpf->rcIn.left; i < lpf->rcIn.right; i++) {
			// builtin method uses NTSC luminance:
			// int gray = CLAMP((299 * data[0] + 587 * data[1] + 114 * data[2] / 1000);
			// here, we take LCD gamma into effect. assume gamma of 2.2
			double gray = g_ramp[0][data[0]] + g_ramp[1][data[1]] + g_ramp[2][data[2]];
			gray = 255.0 * pow(gray, 1.0/2.2);
			if (gray < 0) gray = 0;
			if (gray > 255) gray = 255;
			BYTE bgray = (BYTE)gray;
			*data++ = bgray;
			*data++ = bgray;
			*data++ = bgray;
			data++;
		}
		lpf->rcOut = lpf->rcIn;
	} else {
		// start the loop
		SetRect(&lpf->rcOut, lpf->rcAffect.left, lpf->rcAffect.top - 1, lpf->rcAffect.right, lpf->rcAffect.top);
	}
	// ask for more @ rcIn
	lpf->rcIn = lpf->rcOut;
	OffsetRect(&lpf->rcIn, 0, 1);
	if (bStart) SetRect(&lpf->rcOut, 0, 0, -1, -1);
	if (lpf->rcIn.bottom > lpf->rcAffect.bottom) {
		lpf->rcIn.bottom = lpf->rcAffect.bottom;
	} else {
		lpf->Progress(100 * (lpf->rcIn.top - lpf->rcAffect.top) / (lpf->rcAffect.bottom - lpf->rcAffect.top));
	}
	// no errors here!
	return TRUE;
}

FILTER_MODULE sample_grayscale =
{
	PA_FILTER_VER,
	_T("Stylize"),
	_T("Grayscale"),
	PA_PIXEL_TYPE_RGBA,
	0,
	NULL,
	NULL,
	(LPARAM)&g_ramp[0],
	FilterAbout,
	FilterGrayscaleInit,
	FilterQuit,
	FilterDefaultStartup,
	FilterDefaultConfigure,
	FilterGrayscaleExecute,
	{0},
	NULL,
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



typedef struct RANDOMIZEPARAM
{
	DWORD seed;
	int amount;
	int count;
	int iter;
} RANDOMIZEPARAM, *LPRANDOMIZEPARAM;

RANDOMIZEPARAM g_random = {0, 25};

BOOL FilterRandomizeStartup(LPFILTER_MODULE lpf, int type)
{
	// init seed here
	LPRANDOMIZEPARAM data = (LPRANDOMIZEPARAM)lpf->lParam;
	data->seed = GetTickCount();
	return TRUE;
}

BOOL CALLBACK FilterRandomizeDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	static LPFILTER_MODULE lpf = NULL;
	if (message == WM_INITDIALOG) {
		lpf = (LPFILTER_MODULE)lParam;
		lpf->CreatePreviewPane(hDlg, IDC_PREVIEW, 0, lpf->context);
		LPRANDOMIZEPARAM data = (LPRANDOMIZEPARAM)lpf->lParam;

		int amount = data->amount;
		SendDlgItemMessage(hDlg, IDC_SPIN, UDM_SETRANGE, 0, MAKELONG(10000, 1));
		SendDlgItemMessage(hDlg, IDC_SPIN, UDM_SETPOS, 0, MAKELONG(data->amount, 0));
		SendDlgItemMessage(hDlg, IDC_SPIN, CCM_SETVERSION, COMCTL32_VERSION, 0);
		data->amount = amount;
		SetDlgItemInt(hDlg, IDC_AMOUNT, data->amount, FALSE);

	} else if (message == WM_COMMAND && lpf) {
		LPRANDOMIZEPARAM data = (LPRANDOMIZEPARAM)lpf->lParam;
		if (LOWORD(wParam) == IDOK) {
			data->amount = GetDlgItemInt(hDlg, IDC_AMOUNT, NULL, FALSE);
			EndDialog(hDlg, IDOK);
		} else if (LOWORD(wParam) == IDCANCEL) {
			EndDialog(hDlg, IDCANCEL);
		} else if (LOWORD(wParam) == IDC_AMOUNT && HIWORD(wParam) == EN_CHANGE) {
			BOOL bTrans = FALSE;
			int val = GetDlgItemInt(hDlg, IDC_AMOUNT, &bTrans, FALSE);
			if (bTrans && val) {
				data->amount = val;
				data->seed = GetTickCount();
				if (lpf) lpf->HandlePreviewMessage(hDlg, UWM_PREVIEW_INVALIDATE, 0, 0, lpf->context);
			}
		} else if (lpf) {
			return lpf->HandlePreviewMessage(hDlg, message, wParam, lParam, lpf->context);
		}
	} else if (lpf) {
		return lpf->HandlePreviewMessage(hDlg, message, wParam, lParam, lpf->context);
	}
	return FALSE;
}


BOOL FilterRandomizeConfigure(LPFILTER_MODULE lpf, HWND hwndParent)
{
	// adjust seed here
	LPRANDOMIZEPARAM data = (LPRANDOMIZEPARAM)lpf->lParam;
	return IDOK == DialogBoxParam(lpf->hInstance, MAKEINTRESOURCE(IDD_FILTERRANDOMIZE),
		hwndParent, FilterRandomizeDlgProc, (LPARAM)lpf);
	return TRUE;
}

void RandomRect(LPRECT output, int nw, int nh, LPRECT clamp)
{
	int w = (clamp->right - clamp->left);
	int h = (clamp->bottom - clamp->top);
	int x = w > 1 ? Random() % w : 0;
	int y = h > 1 ? Random() % h : 0;

	output->left = clamp->left + x;
	output->top = clamp->top + y;
	output->right = output->left + nw;
	output->bottom = output->top + nh;

	// ensure that the new rect has the same stride.
	if (output->right > clamp->right) {
		output->left -= output->right - clamp->right;
	}
	if (output->bottom > clamp->bottom) {
		output->bottom -= output->bottom - clamp->bottom;
	}

	// just in case.
	IntersectRect(output, output, clamp);
}

#define MIN(a, b) ((a) < (b) ? (a) : (b))

BOOL FilterRandomizeExecute(LPFILTER_MODULE lpf, BOOL bStart)
{
	if (lpf->Canceled()) return FALSE;

	LPRANDOMIZEPARAM param = (LPRANDOMIZEPARAM)lpf->lParam;

	int width = (lpf->rcAffect.right - lpf->rcAffect.left);
	int height = (lpf->rcAffect.bottom - lpf->rcAffect.top);

	if (bStart) {
		param->count = width * height / (16*16);
		param->count = param->count * param->amount / 100;
		if (!param->count) param->count = 1;
		param->iter = 0;
	}

	if (!IsRectEmpty(&lpf->rcIn) && param->iter < param->count) {
		param->iter++;
		// simple plugin, basically pulls/puts random squares
		RandomRect(&lpf->rcOut, lpf->rcIn.right - lpf->rcIn.left, lpf->rcIn.bottom - lpf->rcIn.top, &lpf->rcAffect);
	}
	// ask for more @ rcIn
	RandomRect(&lpf->rcIn, 1 + (Random() % MIN(64,width)), 1 + (Random() % MIN(64,height)), &lpf->rcAffect);

	if (bStart) SetRect(&lpf->rcOut, 0, 0, -1, -1);
	if (param->iter == param->count) {
		SetRect(&lpf->rcIn, 0, 0, -1, -1);
	} else {
		lpf->Progress(100 * param->iter / param->count);
	}
	// no errors here!
	return TRUE;
}

FILTER_MODULE sample_randomize =
{
	PA_FILTER_VER,
	_T("Distort"),
	_T("Scatter"),
	PA_PIXEL_TYPE_RGBA,
	0,
	NULL,
	NULL,
	(LPARAM)&g_random,
	FilterAbout,
	FilterInit,
	FilterQuit,
	FilterRandomizeStartup,
	FilterRandomizeConfigure,
	FilterRandomizeExecute,
	{0},
	NULL,
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



typedef struct MOSAICPARAM
{
	int amount;
} MOSAICPARAM, *LPMOSAICPARAM;

MOSAICPARAM g_mosaic = {2};


BOOL FilterMosaicStartup(LPFILTER_MODULE lpf, int type)
{
	// init seed here
	LPRANDOMIZEPARAM data = (LPRANDOMIZEPARAM)lpf->lParam;
	data->amount = 2;
	return TRUE;
}
BOOL CALLBACK FilterMosaicDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	static LPFILTER_MODULE lpf = NULL;
	if (message == WM_INITDIALOG) {
		lpf = (LPFILTER_MODULE)lParam;
		lpf->CreatePreviewPane(hDlg, IDC_PREVIEW, 0, lpf->context);
		LPMOSAICPARAM data = (LPMOSAICPARAM)lpf->lParam;

		int amount = data->amount;
		SendDlgItemMessage(hDlg, IDC_SPIN, UDM_SETRANGE, 0, MAKELONG(10000, 2));
		SendDlgItemMessage(hDlg, IDC_SPIN, UDM_SETPOS, 0, MAKELONG(data->amount, 0));
		SendDlgItemMessage(hDlg, IDC_SPIN, CCM_SETVERSION, COMCTL32_VERSION, 0);
		data->amount = amount;
		SetDlgItemInt(hDlg, IDC_AMOUNT, data->amount, FALSE);

	} else if (message == WM_COMMAND && lpf) {
		LPMOSAICPARAM data = (LPMOSAICPARAM)lpf->lParam;
		if (LOWORD(wParam) == IDOK) {
			data->amount = GetDlgItemInt(hDlg, IDC_AMOUNT, NULL, FALSE);
			EndDialog(hDlg, IDOK);
		} else if (LOWORD(wParam) == IDCANCEL) {
			EndDialog(hDlg, IDCANCEL);
		} else if (LOWORD(wParam) == IDC_AMOUNT && HIWORD(wParam) == EN_CHANGE) {
			BOOL bTrans = FALSE;
			int val = GetDlgItemInt(hDlg, IDC_AMOUNT, &bTrans, FALSE);
			if (bTrans && val) {
				data->amount = val;
				if (lpf) lpf->HandlePreviewMessage(hDlg, UWM_PREVIEW_INVALIDATE, 0, 0, lpf->context);
			}
		} else if (lpf) {
			return lpf->HandlePreviewMessage(hDlg, message, wParam, lParam, lpf->context);
		}
	} else if (lpf) {
		return lpf->HandlePreviewMessage(hDlg, message, wParam, lParam, lpf->context);
	}
	return FALSE;
}


BOOL FilterMosaicConfigure(LPFILTER_MODULE lpf, HWND hwndParent)
{
	// adjust seed here
	LPMOSAICPARAM data = (LPMOSAICPARAM)lpf->lParam;
	return IDOK == DialogBoxParam(lpf->hInstance, MAKEINTRESOURCE(IDD_FILTERMOSAIC),
		hwndParent, FilterMosaicDlgProc, (LPARAM)lpf);
	return TRUE;
}

BOOL FilterMosaicExecute(LPFILTER_MODULE lpf, BOOL bStart)
{
	if (lpf->Canceled()) return FALSE;

	LPMOSAICPARAM param = (LPMOSAICPARAM)lpf->lParam;

	if (!IsRectEmpty(&lpf->rcIn)) {
		// process data @ rcIn, one block at a time
		BYTE* data = lpf->pixels.data;
		int i, j, ch;

		int avg[4] = {0};
		for (j = lpf->rcIn.top; j < lpf->rcIn.bottom; j++) {
			for (i = lpf->rcIn.left; i < lpf->rcIn.right; i++) {
				BYTE alpha = data[3];
				for (ch = 0; ch < 3; ch++) {
					avg[ch] += alpha * *data++;
				}
				avg[3] += *data++;
			}
		}

		data = lpf->pixels.data;

		for (ch = 0; avg[3] && ch < 3; ch++) {
			avg[ch] /= avg[3];
		}
		avg[3] /= (lpf->rcIn.right - lpf->rcIn.left) * (lpf->rcIn.bottom - lpf->rcIn.top);

		for (j = lpf->rcIn.top; j < lpf->rcIn.bottom; j++) {
			for (i = lpf->rcIn.left; i < lpf->rcIn.right; i++) {
				for (ch = 0; ch < 4; ch++) {
					*data++ = avg[ch];
				}
			}
		}

		lpf->rcOut = lpf->rcIn;

		OffsetRect(&lpf->rcIn, param->amount, 0);

		if (lpf->rcIn.left >= lpf->rcAffect.right) {
			SetRect(&lpf->rcIn, lpf->rcAffect.left, lpf->rcIn.bottom, lpf->rcAffect.left + param->amount, lpf->rcIn.bottom + param->amount);
		}
	} else {
		// start the loop
		SetRect(&lpf->rcIn, lpf->rcAffect.left, lpf->rcAffect.top, lpf->rcAffect.left + param->amount, lpf->rcAffect.top + param->amount);
	}

	if (lpf->rcIn.bottom > lpf->rcAffect.bottom) {
		lpf->rcIn.bottom = lpf->rcAffect.bottom;
	} else {
		lpf->Progress(100 * (lpf->rcIn.top - lpf->rcAffect.top) / (lpf->rcAffect.bottom - lpf->rcAffect.top));
	}
	// no errors here!
	return TRUE;
}

FILTER_MODULE sample_mosaic =
{
	PA_FILTER_VER,
	_T("Pixelate"),
	_T("Mosaic"),
	PA_PIXEL_TYPE_RGBA,
	0,
	NULL,
	NULL,
	(LPARAM)&g_mosaic,
	FilterAbout,
	FilterInit,
	FilterQuit,
	FilterMosaicStartup,
	FilterMosaicConfigure,
	FilterMosaicExecute,
	{0},
	NULL,
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


BOOL FilterFragmentStartup(LPFILTER_MODULE lpf, int type)
{
	lpf->lParam = 0;
	return TRUE;
}

BOOL FilterFragmentExecute(LPFILTER_MODULE lpf, BOOL bStart)
{
	if (lpf->Canceled()) return FALSE;

	if (!IsRectEmpty(&lpf->rcIn)) {
		if (lpf->lParam == 0) {
			// first pass, copy one row at a time.
			lpf->rcOut = lpf->rcIn;
			// ask for more @ rcIn
			OffsetRect(&lpf->rcIn, 0, 1);
			if (lpf->rcIn.bottom > lpf->rcAffect.bottom) {
				// next pass
				lpf->lParam = 1;
				SetRect(&lpf->rcIn, lpf->rcAffect.left, lpf->rcAffect.top, lpf->rcAffect.right, lpf->rcAffect.top+1);
				lpf->bFillScratch = TRUE;
			} else {
				lpf->Progress(33 * (lpf->rcIn.top - lpf->rcAffect.top) / (lpf->rcAffect.bottom - lpf->rcAffect.top));
			}
		} else if (lpf->lParam == 1) {
			// technically, we could combine the first two passes into *data = *data + *(data+32), but we separate 
			// into multiple passes here to illustrate the use of scratch.data.

			// second pass, mix to the right
			BYTE* data = lpf->pixels.data + 4 * 8;
			BYTE* prev = lpf->scratch.data;
			for (int i = lpf->rcIn.left + 8; i < lpf->rcIn.right; i++) {
				*data++ = (*data + *prev++) / 2;
				*data++ = (*data + *prev++) / 2;
				*data++ = (*data + *prev++) / 2;
				data++;
				prev++;
			}
			lpf->rcOut = lpf->rcIn;
			// ask for more @ rcIn
			OffsetRect(&lpf->rcIn, 0, 1);
			if (lpf->rcIn.bottom > lpf->rcAffect.bottom) {
				// next pass
				lpf->lParam = 2;
				SetRect(&lpf->rcIn, lpf->rcAffect.left, lpf->rcAffect.top, lpf->rcAffect.left+1, lpf->rcAffect.bottom);
				lpf->bFillScratch = TRUE;
			} else {
				lpf->Progress(34 + 33 * (lpf->rcIn.top - lpf->rcAffect.top) / (lpf->rcAffect.bottom - lpf->rcAffect.top));
			}
		} else if (lpf->lParam == 2) {
			// third pass, mix down
			BYTE* out = lpf->pixels.data + 4 * 8;
			BYTE* data = lpf->scratch.data + 4 * 8;
			BYTE* prev = lpf->scratch.data;
			for (int i = lpf->rcIn.top + 8; i < lpf->rcIn.bottom; i++) {
				*out++ = (*data++ + *prev++) / 2;
				*out++ = (*data++ + *prev++) / 2;
				*out++ = (*data++ + *prev++) / 2;
				out++;
				data++;
				prev++;
			}
			lpf->rcOut = lpf->rcIn;
			// ask for more @ rcIn
			OffsetRect(&lpf->rcIn, 1, 0);
			if (lpf->rcIn.bottom > lpf->rcAffect.bottom) {
				// end
				SetRect(&lpf->rcIn, 0, 0, -1, -1);
			} else {
				lpf->Progress(67 + 33 * (lpf->rcIn.left - lpf->rcAffect.left) / (lpf->rcAffect.right - lpf->rcAffect.left));
			}
		}
	} else {
		// start the loop
		SetRect(&lpf->rcIn, lpf->rcAffect.left, lpf->rcAffect.top, lpf->rcAffect.right, lpf->rcAffect.top+1);
	}
	// no errors here!
	return TRUE;
}

FILTER_MODULE sample_fragment =
{
	PA_FILTER_VER,
	_T("Pixelate"),
	_T("Fragment"),
	PA_PIXEL_TYPE_RGBA,
	0,
	NULL,
	NULL,
	(LPARAM)0,
	FilterAbout,
	FilterInit,
	FilterQuit,
	FilterFragmentStartup,
	FilterDefaultConfigure,
	FilterFragmentExecute,
	{0},
	NULL,
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


LPFILTER_MODULE PocketArtistGetFilterModule(HINSTANCE hInst, int iPlug)
{
	if (iPlug == 0) {
		return &sample_grayscale;
	} else if (iPlug == 1) {
		return &sample_randomize;
	} else if (iPlug == 2) {
		return &sample_mosaic;
	} else if (iPlug == 3) {
		return &sample_fragment;
	} else {
		return NULL;
	}
}

