#ifndef PAPLUGINS_H
#define PAPLUGINS_H

#include <windows.h>


#define PA_FILTER_VER	0x1


#define PA_FILTER_STARTUP_INTERACTIVE		0x1
#define PA_FILTER_STARTUP_AUTOREPEAT		0x2

#define PA_PIXEL_TYPE_RGBA	8	// RGB + alpha
#define PA_PIXEL_TYPE_RGB	4	// RGB only
#define PA_PIXEL_TYPE_CA	2	// one channel + alpha
#define PA_PIXEL_TYPE_C		1	// one channel

#define PA_FILTER_FLAG_RGBONLY	1

#define PA_PROPERTY_FGCOLOR	1
#define PA_PROPERTY_BGCOLOR 2
#define PA_PROPERTY_LAYERNAME 3

typedef struct
{
	BYTE* data;
	DWORD type;
	int rows;
	int cols;
	int xstride;	// inc. bytes per col.
	int ystride;	// inc. bytes per row.
} PAPIXELS, *LPPAPIXELS;

typedef struct FILTER_MODULE
{
	int version;
	TCHAR *szCategory;
	TCHAR *szMenu;
	DWORD dwHandleTypes;	// what types can handle. anything else is converted in/out.
	DWORD dwFlags;

	HWND hwndMain;
	HINSTANCE hInstance;
	LPARAM lParam;	// plugin-defined.

	void (*About)(struct FILTER_MODULE*, HWND hwndParent);

	void (*Init)(struct FILTER_MODULE*);
	void (*Quit)(struct FILTER_MODULE*);

	BOOL (*Startup)(struct FILTER_MODULE*, int type);
	BOOL (*Configure)(struct FILTER_MODULE*, HWND hwndParent);

	// this runs repeatedely until RECTs are empty or FALSE is returned.
	BOOL (*Execute)(struct FILTER_MODULE*, BOOL bStart);

	// reserved space for future filter methods
	DWORD dwReserved[16];

	// functions provided by Pocket Artist:

	// utility
	BOOL (*GetProperty)(DWORD prop, int index, int* value, BYTE* data);
	BOOL (*SetProperty)(DWORD prop, int index, int value, BYTE* data, DWORD len);
	BOOL (*Canceled)();
	BOOL (*Progress)(int percent);	// 0 - 100%
	BOOL (*CreatePreviewPane)(HWND hDlg, int nIDPane, int nIDCheck, void* context);
#define UWM_PREVIEW_INVALIDATE	(WM_USER+1000)
	BOOL (*HandlePreviewMessage)(HWND, UINT, WPARAM, LPARAM, void* context);

	// calls Execute, or shows original content.
	void (*Preview)(BOOL bExec, void* context, HDC hdc, RECT* srcRect, int dx, int dy, int zoomin, int zoomout);

	// DATA:
	int width;
	int height;
	int channels;
	RECT rcAffect;
	BOOL hasAlpha;
	BOOL hasMask;
	BOOL alphaLocked;
	BOOL isFloating;
	void* context;	// passed to PA functions

	// fill these on Execute(bStart), and update them as needed when progressing 
	// through the image.
	RECT rcIn;
	RECT rcOut;
	// requests pixels from already produced output too, storing in scrach.
	// used in multipass filters. pixels are copied in after rcOut's pixels have been copied out.
	BOOL bFillScratch;

	// filled by Pocket Artist after Execute(bStart), and updated after each call
	PAPIXELS pixels;
	PAPIXELS scratch;	// where rcInScratch data is placed.


} FILTER_MODULE, *LPFILTER_MODULE;


// rename DLL to AFP, and export "PocketArtistGetFilterModule"
typedef LPFILTER_MODULE (*PocketArtistGetFilterModuleProc)(HINSTANCE, int);


#endif
